package net.sf.javaprinciples.resource;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

/**
 *
 * @author Warwick Slade
 */
public class JsonResourceMetadataDao implements ResourceMetadataDao
{
    GsonBuilder builder;

    public JsonResourceMetadataDao()
    {
        builder = new GsonBuilder();
        builder.setDateFormat("yyyy-MM-dd'T'HH:mm:ss");
    }

    @Override
    public ResourceMetadata fetch(File metadataFile)
    {
        try
        {
            Reader reader = new InputStreamReader(new FileInputStream(metadataFile));
            Gson gson = builder.create();

            return gson.fromJson(reader, ResourceMetadata.class);
        }
        catch (FileNotFoundException e)
        {
            throw new RuntimeException("Invalid metadataFile:" + metadataFile.getAbsolutePath());
        }
    }

    @Override
    public void store(ResourceMetadata metadata, File metadataFile)
    {
        Gson gson = builder.create();

        FileWriter writer = null;
        try
        {
            writer = new FileWriter(metadataFile);
            gson.toJson(metadata, writer);
        }
        catch (IOException e)
        {
            throw new RuntimeException("Invalid metadataFile:" + metadataFile.getAbsolutePath());
        }
        finally
        {
            if (writer != null)
            {
                try
                {
                    writer.close();
                }
                catch (IOException e)
                {
                }
            }
        }

    }
}
