package net.sf.javaprinciples.resource;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author wslade
 * @author kmorgan
 */
@SuppressWarnings("unchecked")
public class MemoryResourceCache implements ResourceCache
{
    private Map<ResourceIdentifier,Resource> cache = new HashMap<ResourceIdentifier,Resource>();

    /**
     * Purges the cache.
     */
    public void clear()
    {
        cache.clear();
    }

    /**
     * Removes a particular Resource from the cache.
     * @param identifier never null
     */
    public void clear(ResourceIdentifier identifier)
    {
        cache.remove(identifier);
    }

    /**
     * @param identifier never null
     * @return true if in memory and NOT expired
     */
    @Override
    public boolean containsResource(ResourceIdentifier identifier)
    {
        // getResource considers expiry.
        return getResource(identifier)!=null;
    }

    /**
     * @param identifier never null
     * @return a resource if it was in memory
     */
    @Override
    public Resource getResource(ResourceIdentifier identifier)
    {
        Resource r = cache.get(identifier);

        // check if a lifespan / expiry date has been set
        if (r!=null && r instanceof AbstractResource)
        {
            AbstractResource ar = (AbstractResource)r;

            // has a lifespan if a 'valid' value is set for validUntil,
            // where 'valid' is positive non-zero
            boolean hasLifespan = ar.getValidUntil()>0;
            if (hasLifespan && ar.getValidUntil()<System.currentTimeMillis())
            {
                // expired! remove from cache and get out of here
                cache.remove(identifier);
                return null;
            }
        }
        return r;
    }

    /**
     * @param identifier never null
     * @param resource resource to add into memory, never null
     */
    @Override
    public void putResource(ResourceIdentifier identifier, Resource resource)
    {
        cache.put(identifier, resource);
    }

}