package net.sf.javaprinciples.resource;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.http.HttpMethod;


/**
 * An implementation of ResourceIdentifierParser implementing the legacy parsing approach.
 * @author rvllewln
 *
 */
public class SinglePathElementResourceIdentifierParser implements ResourceIdentifierParser
{
    private List<String> excludeParameters;
    private boolean excludeAllParameters = true;
    private RequestParser requestParser = new RequestParser();
    
    public void setExcludeParameters(List<String> excludeParameters)
    {
        this.excludeParameters = excludeParameters;
    }
    
    public void setExcludeAllParameters(boolean excludeAllParameters)
    {
        this.excludeAllParameters = excludeAllParameters;
    }
    /**
     * {@inheritDoc}
     */
    @Override
    public ResourceIdentifier parse(HttpServletRequest request) throws ResourceParseException
    {
        List<String> pathElements = requestParser.parseUri(request);
        Map<String, String> queryParameters = requestParser.parseQueryParams(request);
        queryParameters = filterQueryParameters(queryParameters);

        String identifier = "";
        if (!request.getMethod().equals(HttpMethod.POST.toString()))
        {
            if (!pathElements.isEmpty())
            {
                identifier = pathElements.get(pathElements.size() - 1);
            }
        }
        return new DefaultResourceIdentifier(identifier, pathElements, queryParameters);
    }

    /**
     * Remove any parameters that should be excluded.
     * @param queryParameters
     * @return Never null.
     */
    private Map<String, String> filterQueryParameters(Map<String, String> queryParameters)
    {
        Map<String, String> copy = new HashMap<String, String>();
        if (excludeAllParameters)
        {
            return copy;
        }
        
        if (excludeParameters == null || excludeParameters.isEmpty())
        {
            // Nothing todo
            return queryParameters;
        }
        
        for (String key : queryParameters.keySet())
        {
            if (!excludeParameters.contains(key))
            {
                copy.put(key, queryParameters.get(key));
            }
        }
        return copy;
    }
    
}
